__title__ = "Resty-Client"
__version__ = "0.0.5"
__description__ = """RestyClient is a simple, easy-to-use Python library for interacting with REST APIs using Pydantic's 
powerful data validation and deserialization tools."""
